"""
    blc_core wrapper.
"""
from ctypes import *
from ctypes.util import find_library

path=find_library("blc")
lib = CDLL(name=path)

class BlcDim(Structure):
    _fields_ = [("length", c_size_t), ("step", c_size_t)]

class BlcArray:
    lib.blc_array_new.restype = c_void_p
    lib.blc_array_new_void.restype = c_void_p
    lib.blc_array_new_void.argtypes = []
    lib.blc_array_get_type.argtypes = [ c_void_p ]
    lib.blc_array_get_type.restype = c_uint32
    lib.blc_array_get_format.argtypes = [ c_void_p ]
    lib.blc_array_get_format.restype = c_uint32
    lib.blc_array_get_total_length.argtypes = [ c_void_p ]
    lib.blc_array_get_total_length.restype = c_int
    lib.blc_array_get_dims_nb.argtypes = [ c_void_p ]
    lib.blc_array_get_dims_nb.restype = c_int
    lib.blc_array_get_dims.argtypes = [ c_void_p ]
    lib.blc_array_get_data.argtypes = [ c_void_p ]
    lib.blc_array_get_data.restype = c_void_p
    lib.blc_array_delete.argtypes = [ c_void_p ]
    
    types={ 'UIN8': c_uint8, 'INT8': c_int8, 'UI16': c_uint16, 'IN16': c_int16, 'UI32': c_uint32, 'IN32': c_int32, 'UI64': c_uint64, 'IN64': c_int64, 'FL32': c_float, 'FL64': c_double}

    def __init__(self,  data_type=None, format_type=None, dims=None):
        self.type = data_type
        self.format = format_type
        self.dims = dims
        
        if (dims): #Creates
            dim_types = [c_size_t] * len(dims)
            lib.blc_array_new.argtypes = [c_uint32, c_uint32, c_int, *dim_types ]
            
            # Create a new array
            self.pointer = lib.blc_array_new(int.from_bytes(data_type.encode(), "big"), int.from_bytes(format_type.encode(), "big"), len(dims), *dims)

            total_length = lib.blc_array_get_total_length(self.pointer)
        
            # Specify types for array
            self.ctype = BlcArray.types[self.type]
            
            # Specify pointer of data for array
            self.data = cast(lib.blc_array_get_data(self.pointer), POINTER(self.ctype*total_length)).contents
        else:
            self.pointer = lib.blc_array_new_void()
           
    def get_def(self):   
        self.type=lib.blc_array_get_type(self.pointer).to_bytes(4, "big").decode()
        self.ctype = BlcArray.types[self.type]
        self.format=lib.blc_array_get_format(self.pointer)

        
        dims_nb=lib.blc_array_get_dims_nb(self.pointer)
        lib.blc_array_get_dims.restype = POINTER(BlcDim  * dims_nb)
        blc_dims=lib.blc_array_get_dims(self.pointer)
        self.dims=[]
        for i in range(dims_nb):
            self.dims.append(blc_dims[0][i].length)
            
        self.total_length = lib.blc_array_get_total_length(self.pointer)
        self.data = cast(lib.blc_array_get_data(self.pointer), POINTER(self.ctype*self.total_length)).contents
        
    def __del__(self):
        lib.blc_array_delete(self.pointer)

if __name__ == "__main__":
    
    undef_array = BlcArray()

    array = BlcArray('UIN8', 'TEXT', [16, 3])
    array.get_def() #Useless here, only for testing the function
    
    print("type",array.type, "format", array.format, "length", len(array.data))
    print(array.data[:array.total_length])

